import urllib2, urllib
import json
from acct_mgr.web_ui import RegistrationModule
from trac.web.main import IRequestHandler, IRequestFilter
from trac.config import ConfigurationError, Option
from recaptcha.client import captcha

class RecaptchaResponse(object):
    def __init__(self, is_valid, error_code=None):
        self.is_valid = is_valid
        self.error_code = error_code

class RecaptchaRegistrationModule(RegistrationModule):
    public_key = Option('recaptcha', 'public_key',
        doc='The public key given to you from the reCAPTCHA site')
    private_key = Option('recaptcha', 'private_key',
        doc='The private key given to you from the reCAPTCHA site')
    theme = Option('recaptcha', 'theme', default='white',
        doc='Can be red, white (default), blackglass, clean or custom. Please see http://wiki.recaptcha.net/index.php/Theme')

    def check_config(self):
        if not self.public_key or not self.private_key:
            raise ConfigurationError('public_key and private_key needs ' \
                'to be in the [recaptcha] section of your trac.ini file. ' \
                'Get these keys from http://recaptcha.net/')

    def recaptcha_submit (self, recaptcha_response_field,
            private_key,
            remoteip):

        if not (recaptcha_response_field  and
                len (recaptcha_response_field)):
            return RecaptchaResponse (is_valid = False, error_code = 'incorrect-captcha-sol')

        def encode_if_necessary(s):
            if isinstance(s, unicode):
                return s.encode('utf-8')
            return s

        params = urllib.urlencode ({
                'secret': encode_if_necessary(private_key),
                'remoteip' :  encode_if_necessary(remoteip),
                'response' :  encode_if_necessary(recaptcha_response_field),
                })

        request = urllib2.Request (
            url = "https://www.google.com/recaptcha/api/siteverify",
            data = params,
            headers = {
                "Content-type": "application/x-www-form-urlencoded",
                "User-agent": "reCAPTCHA Python"
                }
            )

        httpresp = urllib2.urlopen (request)

	resp = httpresp.read ()
        res = json.loads(resp)
        httpresp.close();

        if (res["success"]):
            return RecaptchaResponse (is_valid=True)
        else:
            return RecaptchaResponse (is_valid=False, error_code = ','.join(res.get("error-codes", ["Unknown"])))

    # IRequestHandler methods
    def process_request(self, req):
        self.check_config()
        action = req.args.get('action')

        if req.method == 'POST' and action == 'create':
            response = self.recaptcha_submit(
		req.args.get('g-recaptcha-response', ""),
                self.private_key,
                req.remote_addr
                )
            if not response.is_valid:
                req.hdf['registration_error'] = 'Captcha incorrect. Please try again.'
		req.hdf['recaptcha_key'] = self.public_key
                req.hdf['recaptcha_theme'] = self.theme
                return "recaptcharegister.cs", None
            else:
                ret = super(RecaptchaRegistrationModule, self).process_request(req)
                h, n = ret
		req.hdf['recaptcha_key'] = self.public_key
                req.hdf['recaptcha_theme'] = self.theme
                #req.hdf['registration_error'] = req.hdf['registration.error'];
                return "recaptcharegister.cs", n
        else:
            ret = super(RecaptchaRegistrationModule, self).process_request(req)
            h, n = ret
	    req.hdf['recaptcha_key'] = self.public_key
            req.hdf['recaptcha_theme'] = self.theme
            return "recaptcharegister.cs", n

    # ITemplateProvider methods
    def get_templates_dirs(self):
        """Return the absolute path of the directory containing the provided
        ClearSilver templates.
        """
        from pkg_resources import resource_filename
        return [resource_filename(__name__, 'templates')]
